from ..core.element import PropAccessor
from .. import elements as el


class combo(PropAccessor):
    def __init__(self, *descriptions, init=None, changed_props=None):
        props = dict()

        if not init:
            init = dict()
        else:
            init = dict(**init)

        # Add {prop_name}_changed to the state, for each prop in
        # `changed_props`
        changed_props_init = (
            {f"{prop_name}_changed": False for prop_name in changed_props}
            if changed_props
            else {}
        )
        changed_props_state = el.state(**changed_props_init)
        descriptions = list(descriptions)
        descriptions.append(changed_props_state)

        def set_prop(prop_name, prop):
            if prop_name in props:
                existing_elem_name = props[prop_name].parent_element.name
                new_elem_name = prop.parent_element.name
                raise Exception(
                    f'Cannot combine a "{existing_elem_name}" with a "{new_elem_name}" '
                    f'because properties like "{prop_name}" appear in both.'
                )
            props[prop_name] = prop
            if prop_name in init:
                prop.init(init[prop_name])
                init.pop(prop_name)

        for description in descriptions:
            if isinstance(description, PropAccessor):
                prop_accessor = description
                for prop in prop_accessor._props.values():
                    set_prop(prop.name, prop)
            else:
                prop_accessor, prop_forwarding_spec = description
                for spec_part in prop_forwarding_spec:
                    if isinstance(spec_part, str):
                        prop_name = spec_part
                        set_prop(prop_name, prop_accessor._props[prop_name])
                    else:
                        prop_name, new_prop_name = spec_part
                        set_prop(new_prop_name, prop_accessor._props[prop_name])

        if init:
            leftover_keys = init.keys()
            raise Exception(f"Unexpected keyword arguments: {', '.join(leftover_keys)}")

        super().__init__(props)

        if changed_props:
            # State in which to store the previous values of the tracked props.
            prev_state = el.state(**{prop_name: None for prop_name in changed_props})

            # If a prop changed (value differs from the previous
            # value), set {prop_name}_changed to True for one frame.
            for prop_name in changed_props:
                prev_value = getattr(prev_state, prop_name)
                current_value = getattr(self, prop_name)

                if prev_value != current_value:
                    setattr(self, f"{prop_name}_changed", True)
                    setattr(prev_state, prop_name, current_value)
                else:
                    setattr(self, f"{prop_name}_changed", False)
