from ..elements.style_utils import lighten
from .. import elements as el
from .combo import combo


class labeled_text_input(combo):
    def __init__(
        self,
        element,
        props_to_forward,
        *label,
        valid=True,
        required=False,
        info_text=None,
        error_text=None,
        **props,
    ):

        self._state = el.state(info_text=info_text, error_text=error_text)

        with el.container(gap=2):
            with el.label_container(gap=2) as self.label_container:
                with el.container(font_size="small", direction="horizontal", gap=2):
                    self.label_text_element = el.text(*label)
                    self.required_text = el.slot()
                with el.container(gap=2) as self.input_container:
                    self.input_element = element(
                        required=required,
                        valid=valid,
                        grow=1,
                        **props,
                    )
                self.helper_text = el.slot()

        render_state = "unedited"
        if self.input_element.edited:
            render_state = "valid" if self.input_element.valid else "invalid"

        info_text = self._state.info_text
        if render_state == "invalid":
            self.label_container.font_color = el.theme().error_color
            self.input_container.background_color = lighten(el.theme().error_color, 100)
            self.input_container.border = el.theme().error_color
            self.input_element.font_color = el.theme().light_font_color
            icon = el.Icons.CircleExclamationSolid
            info_text = self._state.error_text
        else:
            self.label_container.font_color = el.theme().font_color
            self.input_container.background_color = el.theme().surface_color
            self.input_container.border = el.theme().input_border
            self.input_element.font_color = el.theme().font_color
            icon = el.Icons.CircleInfoSolid

        if info_text:
            with self.helper_text:
                el.text(
                    icon,
                    info_text,
                    font_size="x-small",
                    font_color=lighten(el.theme().font_color, 125)
                    if render_state != "invalid"
                    else el.theme().error_color,
                )

        with self.required_text:
            if self.input_element.required and self.label_text_element.label:
                el.text("*", font_color=el.theme().error_color)

        super().__init__(
            (self.label_container, el.input_props),
            (self.input_element, props_to_forward),
            (self.label_text_element, ["label"]),
            self._state,
        )

    @property
    def edited(self):
        return self.input_element.edited
