from nose.tools import assert_raises
from ...core.unittests import simple_test
from ... import elements as el
from ..combo import combo


@simple_test
def test_simple_combo(ctx):
    b = el.button("Hello")
    s = el.state(x=1, y=2)
    c = combo(b, s)

    # Its props are the union of the props in the forwarded elements
    assert set(c._props.keys()) == set(list(b._props.keys()) + list(s._props.keys()))

    assert c.label == b.label
    assert c.x == s.x
    assert c.y == s.y

    # Mutating to the original props modifies the combo props, too
    b.label = "Foo"
    s.x = 100
    s.y = 200

    assert c.label == b.label
    assert c.x == s.x
    assert c.y == s.y

    # Mutating the combo props mutates the originals
    c.label = "Foo"
    c.x = 100
    c.y = 200

    assert c.label == b.label
    assert c.x == s.x
    assert c.y == s.y


@simple_test
def test_combo_forwarding(ctx):
    b = el.button("Hello")
    s = el.state(x=1, y=2)
    c = combo((b, ["label"]), (s, ["x"]))

    assert list(c._props.keys()) == ["label", "x"]
    assert c._props["label"] == b._props["label"]
    assert c._props["x"] == s._props["x"]


@simple_test
def test_combo_init(ctx):
    b = el.button("Hello")
    s = el.state(x=1, y=2)
    c = combo((b, ["label"]), (s, ["x"]), init=dict(label="Goodbye", x=100))

    assert b.label == c.label == "Goodbye"
    assert s.x == c.x == 100

    # Init only some props
    c = combo((b, ["label"]), (s, ["x"]), init=dict(label="Manana"))

    assert b.label == c.label == "Manana"
    assert s.x == c.x == 100

    # Can't init non-forwarded prop
    assert_raises(Exception, combo, (b, ["label"]), (s, ["x"]), init=dict(y=100))


@simple_test
def test_combo_collision(ctx):
    b = el.button("Hello")
    s = el.state(label="Foo")

    # Can't combine elements with colliding props
    assert_raises(Exception, combo, b, s)

    # Rename b's label prop to be `button_label` to avoid the
    # collision
    c = combo(s, (b, [("label", "button_label")]))

    assert c.label == s.label == "Foo"
    assert c.button_label == b.label == "Hello"
