from nose.tools import assert_raises
from ..diff import diff_lists, patch_list, patch_dict


def _test_diff_patch(base, new, expected_diff):
    from copy import deepcopy

    base_copy = deepcopy(base)
    d = diff_lists(base, new)
    assert expected_diff == d
    patch_list(base_copy, d)
    assert base_copy == new


def dom(**kwargs):
    return dict(_dom=True, **kwargs)


def test_diff_patch():
    _test_diff_patch(
        [dom(key=1)],
        [dom(key=1)],
        # Empty diff (equality) is []
        [],
    )
    _test_diff_patch(
        [dom(key=1)],
        [],
        [("delete", 0, 1)],
    )
    _test_diff_patch(
        [],
        [dom(key=1)],
        [("insert", 0, [dom(key=1)])],
    )
    _test_diff_patch(
        [dom(key=1)],
        [dom(key=2)],
        [("delete", 0, 1), ("insert", 0, [dom(key=2)])],
    )
    _test_diff_patch(
        [dom(key=1), dom(key=3)],
        [dom(key=1), dom(key=2), dom(key=3), dom(key=4)],
        [("insert", 1, [dom(key=2)]), ("insert", 3, [dom(key=4)])],
    )
    _test_diff_patch(
        [dom(key=1), dom(key=2), dom(key=3), dom(key=4)],
        [dom(key=1), dom(key=3)],
        [("delete", 1, 1), ("delete", 2, 1)],
    )
    _test_diff_patch(
        [
            dom(key=1),
            dom(
                key=100,
                children=["body"],
                body=[dom(key=1), dom(key=2), dom(key=3), dom(key=4)],
            ),
        ],
        [
            dom(
                key=100,
                stuff=1,
                children=["body"],
                body=[dom(key=1), dom(key=3)],
            )
        ],
        [
            ("delete", 0, 1),
            (
                "dict_diff",
                0,
                [
                    ("subdiff_list", "body", [("delete", 1, 1), ("delete", 2, 1)]),
                    ("update_key", "stuff", 1),
                ],
            ),
        ],
    )
    _test_diff_patch(
        [dom(key=1, y=5)],
        [dom(key=1, p=100)],
        [("dict_diff", 0, [("update_key", "p", 100), ("remove_key", "y")])],
    )
    _test_diff_patch(
        [
            dom(key=1),
            dom(key=2),
            dom(key=3),
            dom(key=4),
            dom(key=5),
            dom(key=6),
        ],
        [dom(key=1), dom(key=3)],
        [("delete", 1, 1), ("delete", 2, 3)],
    )
    _test_diff_patch(
        [dom(key=1), dom(key=3)],
        [dom(key=1), dom(key=2), dom(key=3), dom(key=4), dom(key=5), dom(key=6)],
        [
            ("insert", 1, [dom(key=2)]),
            ("insert", 3, [dom(key=4), dom(key=5), dom(key=6)]),
        ],
    )
    _test_diff_patch(
        [dom(key=1, x=1)],
        [dom(key=1, x=2)],
        [("dict_diff", 0, [("update_key", "x", 2)])],
    )


def test_patch_exceptions():
    # Test unknown chunk names
    assert_raises(Exception, patch_list, [], [("foo", 1)])
    assert_raises(Exception, patch_dict, dict(), [("foo", 1)])
