from ..core.element import Element, Prop
from .label_element import label_join


def parse_fonts(fonts):
    if fonts is None:
        return None

    # We only send webfonts that with urls
    webfonts = []
    for url in fonts.values():
        if url is not None and url not in webfonts:
            webfonts.append(url)
    return webfonts


class head(Element):
    is_singleton = True
    title = Prop("Hyperdiv App")

    # Registered fonts that can be used in hyperdiv
    # Each font has a name and a dict of details
    fonts = Prop(
        {
            "Nunito": None,
            "serif": None,
            "sans-serif": None,
        },
        parse=parse_fonts,
    )


def set_title(*title):
    head().title = label_join(title)


def get_fonts():
    return head().fonts


def import_fonts(*to_import):
    fonts = dict()
    for f in to_import:
        fonts[f[0]] = f[1]

    fonts.update(head().fonts)
    head().fonts = fonts


def remove_fonts(*fontnames):
    fonts = head().fonts.copy()
    for f in fontnames:
        if f in fonts:
            del fonts[f]
    head().fonts = fonts
