from ...core.unittests import simple_test, should_raise
from ..slot import slot
from ..text import text
from ..container import container


@simple_test
def test_slot(ctx):
    t1 = text("1")
    s = slot()
    t2 = text("2")

    assert ctx.collector_stack.current() == [t1, s, t2]

    with s:
        t3 = text("3")
        t4 = text("4")

    assert ctx.collector_stack.current() == [t1, t3, t4, t2]


@simple_test
def test_slot_container(ctx):
    with container() as c:
        t1 = text("1")
        s = slot()
        t2 = text("2")

    assert c.children == [t1, s, t2]

    with s:
        t3 = text("3")
        t4 = text("4")

    assert c.children == [t1, t3, t4, t2]


@simple_test
def test_slot_beginning(ctx):
    with container() as c:
        s = slot()
        t1 = text("1")
        t2 = text("2")

    assert c.children == [s, t1, t2]

    with s:
        t3 = text("3")
        t4 = text("4")

    assert c.children == [t3, t4, t1, t2]


@simple_test
def test_slot_end(ctx):
    with container() as c:
        t1 = text("1")
        t2 = text("2")
        s = slot()

    assert c.children == [t1, t2, s]

    with s:
        t3 = text("3")
        t4 = text("4")

    assert c.children == [t1, t2, t3, t4]


@simple_test
def test_slot_reentry(ctx):
    s = slot()
    with s:
        text("Hi")

    with should_raise():
        with s:
            text("Hello")


@simple_test
def test_slot_is_empty(ctx):
    s = slot()
    assert s.is_empty

    with s:
        text("Hi")

    assert not s.is_empty
